from yta_editor_common.frame.core import FrameBuffer
from yta_editor_common.frame.helper import FrameHelper
from yta_editor_common.frame.core.domain import FrameBufferDomain
from yta_editor_common.frame.registry import register_domain_converter


# TODO: Add the 'requires_dependency' for 'av' (?)
# TODO: Add the 'requires_dependency' for 'numpy' (?)
def media_to_cpu(
    frame: FrameBuffer
) -> FrameBuffer:
    """
    Method to convert from Media (`av.VideoFrame`) to
    CPU (`numpy`).
    """
    size = frame.size
    format = frame.format
    dtype = 'uint8'
    is_normalized = False
    domain = FrameBufferDomain.CPU
    payload = FrameHelper.videoframe_to_ndarray(
        frame = frame.payload,
        do_include_alpha = True
    )

    return FrameBuffer(
        size = size,
        format = format,
        dtype = dtype,
        is_normalized = is_normalized,
        domain = domain,
        payload = payload
    )

    arr = frame.payload.to_ndarray(format = 'rgba')

    return FrameBuffer(
        size = frame.size,
        format = frame.format,
        dtype = 'uint8',
        normalized = False,
        domain = FrameBufferDomain.CPU,
        payload = arr
    )

# TODO: Add the 'requires_dependency' for 'moderngl' (?)
# TODO: Add the 'requires_dependency' for 'numpy' (?)
def gpu_to_cpu(
    frame: FrameBuffer
) -> FrameBuffer:
    """
    Method to convert from GPU (`moderngl.Texture`) to
    CPU (`numpy`).
    """
    w, h = frame.payload.size
    dtype = 'uint8'
    is_normalized = False
    domain = FrameBufferDomain.CPU
    payload, has_alpha = FrameHelper.texture_to_ndarray(
        frame = frame.payload
    )[0]
    format = (
        'rgba'
        if has_alpha else
        'rgb24'
    )

    return FrameBuffer(
        size = (w, h),
        format = format,
        dtype = dtype,
        is_normalized = is_normalized,
        domain = domain,
        payload = payload
    )

    import numpy as np

    tex = frame.payload
    data = np.frombuffer(tex.read(), dtype=np.uint8)
    w, h = tex.size
    arr = data.reshape((h, w, tex.components))
    arr = np.flipud(arr)
    return FrameBuffer(
        size=(w, h),
        format='rgba',
        dtype='uint8',
        is_normalized = False,
        domain=FrameBufferDomain.CPU,
        payload=arr
    )

register_domain_converter(FrameBufferDomain.MEDIA, FrameBufferDomain.CPU, media_to_cpu)
register_domain_converter(FrameBufferDomain.GPU, FrameBufferDomain.CPU, gpu_to_cpu)



"""
The module to include classes related to the
rendering context.
"""
from yta_editor_common.context.gpu_resources import GPUResources
from yta_editor_common.context.config import RenderConfig
from yta_programming.decorators.requires_dependency import requires_dependency
from yta_validation.parameter import ParameterValidator
from typing import Union


class RenderContext:
    """
    A context that will be used along a rendering
    process, to be able to access to different
    resources, providers, etc.
    """

    @property
    @requires_dependency('yta_editor_pools', 'yta_editor_common', 'yta_editor_pools')
    def gpu(
        self
    ) -> 'GPUResources':
        """
        The resources related to GPU.
        """
        if not hasattr(self, '_gpu'):
            self._gpu: GPUResources = GPUResources(self.opengl_context)

        return self._gpu

    def __init__(
        self,
        opengl_context: Union['moderngl.Context', None],
        size: tuple[int, int],
        fps: float,
        audio_fps: int,
        do_use_gpu: bool
    ):
        ParameterValidator.validate_mandatory_instance_of('opengl_context', opengl_context, 'moderngl.Context')
        # TODO: Validate others

        self.config: 'RenderConfig' = RenderConfig(
            size = size,
            fps = fps,
            audio_fps = audio_fps,
            do_use_gpu = do_use_gpu
        )
        """
        The configuration of the rendering process.
        """

    def use_cpu(
        self
    ) -> 'RenderContext':
        """
        Activate the desire to use CPU (and not GPU) by
        setting the `do_use_gpu` parameter to `False`.
        """
        self.config.use_cpu()

        return self

    def use_gpu(
        self
    ) -> 'RenderContext':
        """
        Activate the desire to use GPU (and no CPU) by
        setting the `do_use_gpu` parameter to `True`.
        """
        self.config.use_gpu()

        return self